//
//  MyDocument.m
//  R2O
//
//  Created by Andy Balholm on 6/10/09.
//  Copyright Northland Graphics 2009 . All rights reserved.
//

#import "MyDocument.h"
#import "CompanyInfoController.h"

@implementation MyDocument

- (id)init
{
  self = [super init];
  if (self) 
    {
      companyInfo = [[CompanyInfo alloc] init];
    }
  return self;
}

- (void)makeWindowControllers
{
  NSWindowController * wc = [[[NSWindowController alloc] initWithWindowNibName:@"MyDocument"] autorelease];
  [wc setShouldCloseDocument:YES];
  [self addWindowController:wc];
}

- (IBAction)showCompanyInfoWindow:(id)sender
{
  NSWindowController * wc;
  // See if a company info window is already open.
  NSEnumerator * e = [[self windowControllers] objectEnumerator];
  while (wc = [e nextObject])
    if ([wc isKindOfClass:[CompanyInfoController class]])
      break;
  // Otherwise, create a new one.
  if (wc == nil)
    {
      wc = [[[CompanyInfoController alloc] init] autorelease];
      [self addWindowController:wc];
    }
  [wc showWindow:self];
}

- (NSData *)dataRepresentationOfType:(NSString *)typeName
{
  NSMutableData * data = [NSMutableData data];
  NSKeyedArchiver * archiver = [[NSKeyedArchiver alloc] initForWritingWithMutableData:data];
  [archiver encodeObject:[self companyInfo] forKey:@"companyInfo"];
  // TODO: contracts
  [archiver finishEncoding];
  [archiver release];
  
  return data;
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)typeName
{
  NSKeyedUnarchiver * unarchiver = [[NSKeyedUnarchiver alloc] initForReadingWithData:data];
  CompanyInfo * ci = [unarchiver decodeObjectForKey:@"companyInfo"];
  if (!ci)
    ci = [[[CompanyInfo alloc] init] autorelease];
  [self setCompanyInfo:ci];
  // TODO: contracts
  [unarchiver finishDecoding];
  [unarchiver release];
  
  return YES;
}

- (void)setValue:(id)value forKey:(NSString *)key ofObject:(id)model
{
  id oldValue = [model valueForKey:key];
  if (![value isEqual:oldValue])
    {
      [[[self undoManager] prepareWithInvocationTarget:self] setValue:oldValue forKey:key ofObject:model];
      [model setValue:value forKey:key];
    }
}

- (CompanyInfo *)companyInfo
{
  return companyInfo;
}

- (void)setCompanyInfo:(CompanyInfo *)c
{
  [c retain];
  [companyInfo release];
  companyInfo = c;
}

- (void)dealloc
{
  [self setCompanyInfo:nil];
  
  [super dealloc];
}

@end
