//
//  WindowAndObjectController.m
//  R₂O
//
//  Created by Andy Balholm on 6/13/09.
//  Copyright 2009 Northland Graphics. All rights reserved.
//

#import "WindowAndObjectController.h"
#import "MyDocument.h"


@implementation WindowAndObjectController

// Setup that is shared among initializers
- (void)_doSetup
{
  activeEditors = [[NSMutableArray alloc] init];
}

- (id)initWithWindow:(NSWindow *)window
{
  self = [super initWithWindow:window];
  if (self)
    [self _doSetup];
  return self;
}

- (id)initWithWindowNibName:(NSString *)windowNibName
{
  self = [super initWithWindowNibName:windowNibName];
  if (self)
    [self _doSetup];
  return self;
}

- (id)initWithWindowNibName:(NSString *)windowNibName owner:(id)owner
{
  self = [super initWithWindowNibName:windowNibName owner:owner];
  if (self)
    [self _doSetup];
  return self;
}

- (id)initWithWindowNibPath:(NSString *)windowNibPath owner:(id)owner
{
  self = [super initWithWindowNibPath:windowNibPath owner:owner];
  if (self)
    [self _doSetup];
  return self;
}

- (BOOL)windowShouldClose:(id)window
{
  // Before window closes, record changes that have been made but not committed.
  return [self commitEditing];
}

- (BOOL)commitEditing
{
  if ([activeEditors count] == 0)
    return YES;
  NSEnumerator * e = [activeEditors objectEnumerator];
  id editor;
  while (editor = [e nextObject])
    if ([editor commitEditing] == NO)
      return NO;
  return YES;
}

- (void)discardEditing
{
  if ([activeEditors count] == 0)
    return;
  NSEnumerator * e = [activeEditors objectEnumerator];
  id editor;
  while (editor = [e nextObject])
    [editor discardEditing];
}

- (void)objectDidBeginEditing:(id)editor
{
  [activeEditors addObject:editor];
}

- (void)objectDidEndEditing:(id)editor
{
  [activeEditors removeObject:editor];
}

- (BOOL)isEditing
{
  return [activeEditors count] > 0;
}

- (void)setValue:(id)value forKeyPath:(NSString *)keyPath
{
  // If a change is being made to content, register it for undo.
  if ([keyPath hasPrefix:@"content."])
    {
      unsigned lastDot = [keyPath rangeOfString:@"." options:NSBackwardsSearch].location;
      NSString * baseKeyPath = [keyPath substringToIndex:lastDot];
      NSString * lastKey = [keyPath substringFromIndex:lastDot + 1];
      id affectedObject = [self valueForKeyPath:baseKeyPath];
      id oldValue = [affectedObject valueForKey:lastKey];
      MyDocument * document = [self document];
      [[[document undoManager] prepareWithInvocationTarget:document] setValue:oldValue forKey:lastKey ofObject:affectedObject];
      [affectedObject setValue:value forKey:lastKey];
    }
  else
    [super setValue:value forKeyPath:keyPath];
}

- (id)content
{
  return content;
}

- (void)setContent:(id)c
{
  [c retain];
  [content release];
  content = c;
}

- (void)dealloc
{
  [activeEditors release];
  [content release];
  
  [super dealloc];
}

@end
